@@ -496,48 +496,6 @@ def is_positive_definite(B):
         return False
 
 
-def c_nu_from_ps21_fct(ps3d_fct, freqs, uv_bins, delta_kpar=0.05, normalize=True):
-    ''' Return frequency-frequency covariance for the given baselines bins and frequency 
-        given the spherically averaged power-spectra P(k) (and not delta(k)). 
-        Assume isotropie of the signal, which is true to some extend for the 21-cm signal.
-
-        See https://arxiv.org/abs/astro-ph/0605546'''
-    z = psutil.freq_to_z(freqs.mean())
-
-    uu = np.array(uv_bins).mean(axis=1)
-    k_per = psutil.l_to_k(2 * np.pi * uu, z)
-
-    r = psutil.angular_to_comoving_distance(psutil.freq_to_z(freqs))
-    delta_r = abs(r - r[0])
-
-    d_delta_r = np.diff(delta_r).mean()
-    k_par_max = np.pi  / d_delta_r
-    k_par = np.arange(0, k_par_max, k_par_max / len(r))
-
-    y = ps3d_fct(np.sqrt(k_per[None, :, None] ** 2 + k_par[:, None, None] ** 2)) * np.cos(delta_r[None, None, :] * k_par[:, None, None])
-    c_nu_nu = trapezoid(y, k_par, axis=0)
-
-    if normalize:
-        c_nu_nu = c_nu_nu / c_nu_nu.max()
-
-    return c_nu_nu
-
-
-def c_nu_from_ps21(k_mean, ps3d, freqs, uv_bins, delta_kpar=0.05, normalize=True):
-    '''Return frequency-frequency covariance for the given baselines bins and frequency 
-       given the spherically averaged power-spectra delta(k) (and not P(k)).
-       Assume isotropic of the signal, which is true to some extend for the 21-cm signal.
-
-       See https://arxiv.org/abs/astro-ph/0605546'''
-    def ps_fct(k): return 1 / k ** 3 * scipy.interpolate.interp1d(k_mean, ps3d,
-                                                                  bounds_error=False,
-                                                                  kind='quadratic',
-                                                                  fill_value='extrapolate')(k)
-
-    return np.clip(c_nu_from_ps21_fct(ps_fct, freqs, uv_bins, delta_kpar=delta_kpar,
-                                      normalize=normalize), 1e-20, 1e20)
-
-
 class CovarianceGenerator(object):
     
     @lru_cache(maxsize=10)
@@ -567,6 +525,8 @@ class CovarianceGenerator(object):
         if normalize:
             c_nu_nu = c_nu_nu / c_nu_nu.max()
 
+        c_nu_nu[~np.isfinite(c_nu_nu)] = 0
+
         return c_nu_nu
 
     def c_nu_from_ps21(self, k_mean, ps3d, normalize=True):
@@ -583,7 +543,7 @@ class CovarianceGenerator(object):
                 return 1 / k ** 3 * scipy.interpolate.interp1d(k_mean, ps3d, bounds_error=False, kind='quadratic',
                                                                fill_value='extrapolate')(k)
 
-        return np.clip(self.c_nu_from_ps21_fct(ps_fct, normalize=normalize), 1e-20, 1e20)
+        return self.c_nu_from_ps21_fct(ps_fct, normalize=normalize)
 
 
 @lru_cache(maxsize=10)
@@ -612,7 +572,8 @@ def make_new_vis_cube_from_ps(k_mean, ps3d, res, n_pix, freqs, umin, umax, uv_bi
 
         See also c_nu_from_ps21() and make_new_vis_cube() '''
     uv_bins = get_uv_bins(umin, umax, uv_bins_du)
-    c_nu_nu = c_nu_from_ps21(k_mean, ps3d, freqs, uv_bins, delta_kpar=0.025, normalize=False)
+    cov_gen = get_cached_cov_gen(tuple(freqs), tuple(uv_bins))
+    c_nu_nu = cov_gen.c_nu_from_ps21(k_mean, ps3d, normalize=False)
     K = c_nu_to_K(freqs, c_nu_nu)
 
     z = psutil.freq_to_z(freqs.mean())
@@ -1788,6 +1749,7 @@ class UltraNestNestedSampler(AbstractSampler):
         AbstractSampler.__init__(self, gp)
         self.sampler = ultranest.ReactiveNestedSampler(self.get_parameter_names(), self.log_marginal_likelihood,
                                                        self.prior_transform, **kargs_ultranest)
+        self.sampler_results = None
 
     def run(self, live_update=False, **kargs_ultranest):
         viz_callback = 'auto'
@@ -1811,8 +1773,14 @@ class UltraNestNestedSampler(AbstractSampler):
         d['logl'] = res['weighted_samples']['logl']
         d['logz'] = [res['logz'],]
         d['logzerr'] = res['logzerr']
+
+        self.sampler_results = d
         
-        return SamplerResult(self.gp, NestedSamples(d))
+        return self.get_result()
+
+    def get_result(self):
+        return SamplerResult(self.gp, NestedSamples(self.sampler_results))
+
 
 
 class MLGPRConfigFile(settings.BaseSettings):
@@ -1914,7 +1882,7 @@ class MLGPRResult(object):
                 sampler = emcee.backends.HDFBackend(mcmc_file)
                 samples = MCMCSamples(sampler.get_chain(), sampler.get_log_prob(), n_burn=config.mcmc.n_burn,
                                       autocorr_time=sampler.get_autocorr_time(tol=0))
-        elif sampler_method == 'nested':
+        elif sampler_method in ['nested', 'ultranest']:
             samples_file = os.path.join(save_dir, save_name + '.nested_samples.h5')
             samples = NestedSamples.load(samples_file)
         else:
@@ -1932,8 +1900,8 @@ class MLGPRResult(object):
         self.sampler_result.samples.save(f'{save_dir}/{save_name}.{self.sampler_result.samples.sampler_method}_samples.h5')
 
     def get_ps(self, ps_gen, kbins, kern_name, n_pick=50, subtract_from=None):
-        return self.sampler_result.get_ps_stack(ps_gen, kbins, self.config.mcmc.n_burn, kern_name=kern_name,
-                                                subtract_from=subtract_from)
+        return self.sampler_result.get_ps_stack(ps_gen, kbins, kern_name=kern_name,
+                                                subtract_from=subtract_from, n_pick=n_pick)
 
     def get_ps_fg(self, ps_gen, kbins, n_pick=50):
         return self.get_ps(ps_gen, kbins, 'fg*', n_pick)
