@@ -193,6 +193,7 @@ class HERA(Telescope):
         self.sep = sep
 
     def get_stat_pos(self):
+        # Taken from https://github.com/HERA-Team/hera_sim/blob/main/hera_sim/antpos.py. Credit: HERA team.
         positions = []
         for row in range(self.hex_num - 1, -self.hex_num + self.split_core, -1):
             # adding self.split_core deletes a row if it's true
@@ -481,12 +482,12 @@ class TelescopeSimu(object):
 
         return TelescopGridded(w_cube, self)
 
-    def image_gridding(self, fov, oversampling_factor=4, min_weight=10):
+    def image_gridding(self, fov_deg, oversampling_factor=4, min_weight=10):
         from ps_eor import psutil, datacube
 
         uu_meter, vv_meter, _ = self.simu_uv()
 
-        du = 1 / np.radians(fov)
+        du = 1 / np.radians(fov_deg)
         res = 1 / (oversampling_factor * self.umax)
         n_u = int(np.ceil(1 / (res * du)))
         shape = (n_u, n_u)
