@@ -206,7 +206,7 @@ class MultiNightsPowerSpectraGenerator(object):
 
 class PowerSpectraConfig(psutil.SimpleConfig):
 
-    def __init__(self, el=None, window_fct='hanning', ft_method='nudft', ps2d_pos_only=True):
+    def __init__(self, el=None, window_fct='hann', ft_method='nudft', ps2d_pos_only=True):
         """Power spectra estimator configuration
 
         Args:
@@ -220,7 +220,7 @@ class PowerSpectraConfig(psutil.SimpleConfig):
         """
         psutil.SimpleConfig.__init__(self)
         # Delay transform config
-        self.add('window_fct', 'hanning', str)
+        self.add('window_fct', 'hann', str)
         self.add('ft_method', 'nudft', str)
         self.add('rmean_freqs', False, bool)
         self.add('ps2d_pos_only', True, bool)
@@ -620,9 +620,11 @@ class BasePowerSpectra(object):
     def delay_transform(self, data_cube):
         ''' Delay transform data_cube '''
         if isinstance(data_cube, datacube.NoiseStdCube):
-            delay, dft_cube = psutil.lssa_diag_cov(data_cube.freqs, data_cube.data ** 2, 
+            delay, dft_cube_sq = psutil.lssa_diag_cov(data_cube.freqs, data_cube.data ** 2, 
                                                    M=self.eor.M, dx=self.config.df)
-            dft_cube = dft_cube ** .5
+            # Take into account the spectral tapering window function which is not
+            # applied here, but is still corrected for later in the code. 
+            dft_cube = (dft_cube_sq / self.get_window_fct_norm(data_cube)) ** .5
         else:
             weights = self.get_weights(data_cube, True)
             window = self.get_window_fct(data_cube)
@@ -738,7 +740,12 @@ class PowerSpectraCart(BasePowerSpectra):
         f_delay, dft_cube1 = self.delay_transform(data_cube1)
         f_delay, dft_cube2 = self.delay_transform(data_cube2)
 
-        weight_cube = self.get_weights(data_cube1, weighted, delay_transform=True)
+        w1 = self.get_weights(data_cube1, weighted, delay_transform=True)
+        w2 = self.get_weights(data_cube2, weighted, delay_transform=True)
+        if w1 is None or w2 is None:
+            weight_cube = None
+        else:
+            weight_cube = np.sqrt(w1 * w2)
 
         delay, ps2d, ps2d_err, n_eff, ps2d_w = pscart.get_2d_cross_power_spectra(f_delay, dft_cube1, dft_cube2,
                                                                                  data_cube1.uu,
@@ -796,7 +803,12 @@ class PowerSpectraCart(BasePowerSpectra):
         data_cube2 = self.eor.get_slice(data_cube2)
         ps_norm = self.get_ps_norm(data_cube1)
 
-        weight_cube = self.get_weights(data_cube1, weighted)
+        w1 = self.get_weights(data_cube1, weighted)
+        w2 = self.get_weights(data_cube2, weighted)
+        if w1 is None or w2 is None:
+            weight_cube = None
+        else:
+            weight_cube = np.sqrt(w1 * w2)
 
         ps, ps_err, n_eff, ps_w = pscart.get_cross_power_spectra(data_cube1.data, data_cube2.data, data_cube1.uu,
                                                                  data_cube1.vv, self.el, weight_cube=weight_cube)
@@ -877,7 +889,12 @@ class PowerSpectraCart(BasePowerSpectra):
         _, dft_cube2 = self.delay_transform(data_cube1)
         _, dft_cube1 = self.delay_transform(data_cube2)
 
-        weight_cube = self.get_weights(data_cube1, weighted, delay_transform=True)
+        w1 = self.get_weights(data_cube1, weighted, delay_transform=True)
+        w2 = self.get_weights(data_cube2, weighted, delay_transform=True)
+        if w1 is None or w2 is None:
+            weight_cube = None
+        else:
+            weight_cube = np.sqrt(w1 * w2)
 
         k_per_full = psutil.l_to_k(data_cube1.ru * 2 * np.pi, self.z)
 
