@@ -2,20 +2,11 @@
 #
 # Authors: F.Mertens
 
-from __future__ import division
-from __future__ import print_function
-from __future__ import absolute_import
-from io import open
-
-try:
-    from itertools import izip as zip
-except ImportError:  # will be 3.x series
-    pass
-
 import re
 import os
 import sys
 import time
+import copy
 import functools
 import itertools
 import configparser
@@ -26,6 +17,8 @@ from scipy.signal import get_window
 
 import numpy as np
 
+import h5py
+
 
 NUM_POOL = int(os.environ.get('OMP_NUM_THREADS', 2))
 
@@ -216,6 +209,12 @@ class SimpleConfig(object):
         items = dict(config.items(section))
         self.parse_dict(items, check_existing=check_existing)
 
+    def copy(self):
+        new = self.__class__()
+        new.config = copy.deepcopy(self.config)
+
+        return new
+
 
 def bin_data(x, y, nbins, stat='mean'):
     """Use stats.binned_statistic to bin data
@@ -1401,3 +1400,23 @@ def get_cov_r(cov_matrix, dx, bins):
     cov_m, bins, _ = stats.binned_statistic(r, cov_matrix, bins=bins)
     m_bins = np.mean(np.vstack([bins[1:], bins[:-1]]), axis=0)
     return m_bins, cov_m
+
+
+def save_dict_to_h5(filename, d):
+    with h5py.File(filename, 'w') as hf:
+        for name, value in d.items():
+            hf.create_dataset(name, data=value)
+
+
+def load_dict_from_h5(filename):
+    d = dict()
+    with h5py.File(filename, 'r') as hf:
+        for name, value in hf.items():
+            if value.ndim > 0:
+                d[name] = value[:]
+            else:
+                if value.dtype == 'object':
+                    d[name] = value.asstr()[()]
+                else:
+                    d[name] = value[()]
+    return d
